from hashlib import sha256
import json
import logging
from typing import List, Generator
import os


current_file_path = os.path.dirname(os.path.realpath(__file__))
cache_file = os.path.join(current_file_path, "citrixagent", "config", "log_cache.json")


def _hash(line: str):
    return sha256(line.encode()).hexdigest()


def _tail(file_name: str, num_lines: int):
    # Very naive implementation, I know the files are less than 10MB
    with open(file_name, "r") as f:
        return [line.strip() for line in f.readlines()[-num_lines:]]


def _get_cache() -> List[str]:
    try:
        with open(cache_file, "r") as f:
            return json.load(f)
    except Exception:
        return []


def _update_cache(lines: List[str]):
    with open(cache_file, "w") as f:
        json.dump(lines, f)


def get_lines_from_file(file_name: str, num_lines=1000) -> Generator[str, None, None]:
    current_lines_hash = []
    previous_lines_hash = _get_cache()
    for line in _tail(file_name, num_lines):
        hash_line = _hash(line)
        if hash_line not in previous_lines_hash:
            yield line
        current_lines_hash.append(hash_line)
    _update_cache(current_lines_hash)


def main():

    logger = logging.getLogger(__name__)
    st = logging.StreamHandler()
    logger.setLevel(logging.DEBUG)
    fmt = logging.Formatter("%(asctime)s - %(levelname)s - %(funcName)s - %(message)s ")
    st.setFormatter(fmt)
    logger.addHandler(st)

    for line in get_lines_from_file("D:/projects/go/citrix-agent-go/citrix-agent.log", 10000):
        print(line)


if __name__ == "__main__":
    main()
